/*******************************************************************************
 *  Imixs IX Workflow Technology
 *  Copyright (C) 2001, 2008 Imixs Software Solutions GmbH,  
 *  http://www.imixs.com
 *  
 *  This program is free software; you can redistribute it and/or 
 *  modify it under the terms of the GNU General Public License 
 *  as published by the Free Software Foundation; either version 2 
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *  General Public License for more details.
 *  
 *  You can receive a copy of the GNU General Public
 *  License at http://www.gnu.org/licenses/gpl.html
 *  
 *  Contributors:  
 *  	Imixs Software Solutions GmbH - initial API and implementation
 *  	Ralph Soika
 *******************************************************************************/
package org.imixs.workflow.jee.soap;

import java.rmi.RemoteException;
import java.util.Collection;

import javax.ejb.EJB;
import javax.jws.WebMethod;
import javax.jws.WebService;

import org.imixs.workflow.ItemCollection;
import org.imixs.workflow.jee.ejb.EntityService;
import org.imixs.workflow.jee.ejb.ModelService;
import org.imixs.workflow.webservices.soap.WorkflowModelService;
import org.imixs.workflow.xml.EntityCollection;
import org.imixs.workflow.xml.XMLItemCollection;
import org.imixs.workflow.xml.XMLItemCollectionAdapter;

/**
 * The WebService Implementation class uses the corresponding stateless session
 * ejb Implementation class. This class must match the WSDL file
 * ix_modelservice.wsdl. Therefore the necessary elements
 * (name,targetNamespace,serviceName,portName) are named in the WebService
 * annotation. The WSDL file have to be supported in the META-INF/wsdl
 * directory. For basic authentification this EJB is defined additional in the
 * sun-ejb-jar.xml
 * 
 * The Update Methods now verify the $modelversion. Model Entries with same
 * version number will be updated. To remove an specific Model Version use
 * org.imixs.workflow.jee.adminclient.web
 * 
 * @author Ralph Soika
 * @version 0.0.1
 * 
 */
/*
@DeclareRoles( { "org.imixs.ACCESSLEVEL.NOACCESS",
		"org.imixs.ACCESSLEVEL.READERACCESS",
		"org.imixs.ACCESSLEVEL.AUTHORACCESS",
		"org.imixs.ACCESSLEVEL.EDITORACCESS",
		"org.imixs.ACCESSLEVEL.MANAGERACCESS" })
@RolesAllowed( { "org.imixs.ACCESSLEVEL.NOACCESS",
		"org.imixs.ACCESSLEVEL.READERACCESS",
		"org.imixs.ACCESSLEVEL.AUTHORACCESS",
		"org.imixs.ACCESSLEVEL.EDITORACCESS",
		"org.imixs.ACCESSLEVEL.MANAGERACCESS" })
		*/
//@Stateless
//@Remote(org.imixs.workflow.jee.ws.ModelWebService.class)
//@WebService(name = "WorkflowModelService", targetNamespace = "http://imixs.org/workflow/services", serviceName = "WorkflowModelService", portName = "WorkflowModelServicePort", wsdlLocation = "ix_modelservice.wsdl")
@WebService			
public class ModelWebServiceBean implements WorkflowModelService {

	@EJB  
	EntityService entityService;

	@EJB
	ModelService modelService;
 
//	@Resource 
//	SessionContext ctx;

	@WebMethod
	public EntityCollection getEnvironmentList() throws Exception {
		return XMLItemCollectionAdapter.putCollection(modelService
				.getEnvironmentEntityList());
	}

	@WebMethod
	public EntityCollection getEnvironmentListByVersion( String modelversion)
			throws Exception {

		return XMLItemCollectionAdapter.putCollection(modelService
				.getEnvironmentEntityListByVersion(modelversion));
	}

	@WebMethod
	public EntityCollection getProcessList() throws Exception {
		Collection<ItemCollection> col = modelService.getProcessEntityList();
		return XMLItemCollectionAdapter.putCollection(col);
	}

	@WebMethod
	public EntityCollection getProcessListByVersion( String modelversion)
			throws Exception {

		Collection<ItemCollection> col = modelService
				.getProcessEntityListByVersion(modelversion);
		return XMLItemCollectionAdapter.putCollection(col);
	}

	@WebMethod
	public EntityCollection getActivityList( int processid)
			throws Exception {
		Collection<ItemCollection> col = modelService
				.getActivityEntityList(processid);
		return XMLItemCollectionAdapter.putCollection(col);
	}

	@WebMethod
	public EntityCollection getActivityListByVersion( int processid, String modelversion)
			throws Exception {
		Collection<ItemCollection> col = modelService
				.getActivityEntityListByVersion(processid, modelversion);
		return XMLItemCollectionAdapter.putCollection(col);
	}

	/**
	 * This method removes a specific ModelVersion defined by param
	 * 'modelversion'. If modelVersion is null the method will remove all models
	 */
	@WebMethod
	public void removeModelVersion( String modelversion)
			throws RemoteException, Exception {
		modelService.removeModelVersion(modelversion);
	}

	/**
	 * This method updates a given ProcessList for a specific ModelVersion. The
	 * Method overrides existing Entities. It did not remove the list. To remove
	 * a model use method removeModelVersion
	 */
	@WebMethod
	public void updateProcessList(EntityCollection processlist)
			throws RemoteException, Exception {

		// create new processEnties
		for (int i = 0; i < processlist.getEntity().length; i++) {
			XMLItemCollection ent = processlist.getEntity()[i];
			ItemCollection ic = XMLItemCollectionAdapter.getItemCollection(ent);
			modelService.saveProcessEntity(ic);
		}
	}

	/**
	 * This method updates a given ActivityList for a specific ModelVersion. The
	 * Method overrides existing Entities. It did not remove the list. To remove
	 * a model use method removeModelVersion
	 */
	@WebMethod
	public void updateActivityList(int processid,EntityCollection activitylist)
			throws RemoteException, Exception {

		// create new entities
		for (int i = 0; i < activitylist.getEntity().length; i++) {
			XMLItemCollection ent = activitylist.getEntity()[i];
			ItemCollection ic = XMLItemCollectionAdapter.getItemCollection(ent);
			modelService.saveActivityEntity(ic);
		}
 
	}

	/**
	 * This method updates a given EnvironmentList for a specific ModelVersion.
	 * The Method overrides existing Entities. It did not remove the list. To
	 * remove a model use method removeModelVersion
	 */
	@WebMethod
	public void updateEnvironmentList( EntityCollection environmentlist)
			throws RemoteException, Exception {

		// create new entities
		for (int i = 0; i < environmentlist.getEntity().length; i++) {
			XMLItemCollection ent = environmentlist.getEntity()[i];
			ItemCollection ic = XMLItemCollectionAdapter.getItemCollection(ent);
			modelService.saveEnvironmentEntity(ic);
		}
	}

	
}
