/*******************************************************************************
 *  Imixs IX Workflow Technology
 *  Copyright (C) 2001, 2008 Imixs Software Solutions GmbH,  
 *  http://www.imixs.com
 *  
 *  This program is free software; you can redistribute it and/or 
 *  modify it under the terms of the GNU General Public License 
 *  as published by the Free Software Foundation; either version 2 
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *  General Public License for more details.
 *  
 *  You can receive a copy of the GNU General Public
 *  License at http://www.gnu.org/licenses/gpl.html
 *  
 *  Contributors:  
 *  	Imixs Software Solutions GmbH - initial API and implementation
 *  	Ralph Soika
 *******************************************************************************/
package org.imixs.workflow.jee.soap;

import java.util.Collection;
import java.util.Iterator;

import javax.ejb.EJB;
import javax.jws.WebMethod;
import javax.jws.WebService;

import org.imixs.workflow.ItemCollection;
import org.imixs.workflow.jee.ejb.EntityService;
import org.imixs.workflow.jee.ejb.ReportService;
import org.imixs.workflow.webservices.soap.WorkflowReportService;
import org.imixs.workflow.xml.EntityCollection;
import org.imixs.workflow.xml.XMLItemCollection;
import org.imixs.workflow.xml.XMLItemCollectionAdapter;

/**
 * The WebService Implementation class uses the corresponding stateless session
 * ejb Implementation class. This class must match the corresponding WSDL file.
 * Therefore the necessary elements (name,targetNamespace,serviceName,portName)
 * are named in the WebService annotation. The WSDL file have to be supported in
 * the META-INF/wsdl directory. For basic authentification this EJB is defined
 * additional in the sun-ejb-jar.xml
 * 
 * @author Ralph Soika
 * @version 0.0.1
 */

/*
 * @DeclareRoles( { "org.imixs.ACCESSLEVEL.NOACCESS",
 * "org.imixs.ACCESSLEVEL.READERACCESS", "org.imixs.ACCESSLEVEL.AUTHORACCESS",
 * "org.imixs.ACCESSLEVEL.EDITORACCESS", "org.imixs.ACCESSLEVEL.MANAGERACCESS"
 * })
 * 
 * @RolesAllowed( { "org.imixs.ACCESSLEVEL.NOACCESS",
 * "org.imixs.ACCESSLEVEL.READERACCESS", "org.imixs.ACCESSLEVEL.AUTHORACCESS",
 * "org.imixs.ACCESSLEVEL.EDITORACCESS", "org.imixs.ACCESSLEVEL.MANAGERACCESS"
 * })
 */
// @Stateless
// @Remote(org.imixs.workflow.jee.ws.ReportWebService.class)
// @WebService(name = "WorkflowReportService", targetNamespace =
// "http://imixs.org/workflow/services", serviceName = "WorkflowReportService",
// portName = "WorkflowReportServicePort", wsdlLocation =
// "ix_reportservice.wsdl")

@WebService
public class ReportWebServiceBean implements WorkflowReportService {

	@EJB
	EntityService entityService;

	@EJB
	ReportService reportService;

	@WebMethod
	public XMLItemCollection getReport(String reportid) throws Exception {

		ItemCollection itemCol = reportService.getReport(reportid);
		return XMLItemCollectionAdapter.putItemCollection(itemCol);

	}

	@WebMethod
	public EntityCollection processQuery(String reportid) throws Exception {

		Collection<ItemCollection> col = reportService.processReport(reportid);

		return XMLItemCollectionAdapter.putCollection(col);
	}

	@WebMethod
	public void updateReport(XMLItemCollection report) throws Exception {

		ItemCollection itemCol = XMLItemCollectionAdapter
				.getItemCollection(report);
		reportService.updateReport(itemCol);

	}

	@WebMethod
	public EntityCollection getReportList(int startpos, int count)
			throws Exception {

		Collection<ItemCollection> reportCol = reportService.getReportList(
				startpos, count);
		return XMLItemCollectionAdapter.putCollection(reportCol);
	}

	@WebMethod
	public void updateReportList(EntityCollection reportlist) throws Exception {

		// remove all existing entities
		String sQuery = null;
		sQuery = "SELECT";
		sQuery += " wi FROM Entity as wi " + "JOIN wi.textItems as t "
				+ "WHERE t.itemName = 'type' and t.itemValue = 'QueryEntity'";

		Collection<ItemCollection> col = entityService.findAllEntities(sQuery,
				0, -1);
		Iterator<ItemCollection> it = col.iterator();
		while (it.hasNext()) {
			entityService.remove(it.next());
		}

		// create new entities
		for (int i = 0; i < reportlist.getEntity().length; i++) {
			XMLItemCollection ent = reportlist.getEntity()[i];
			ItemCollection ic = XMLItemCollectionAdapter.getItemCollection(ent);
			reportService.updateReport(ic);
		}

	}

	

}
