/*******************************************************************************
 *  Imixs IX Workflow Technology
 *  Copyright (C) 2001, 2008 Imixs Software Solutions GmbH,  
 *  http://www.imixs.com
 *  
 *  This program is free software; you can redistribute it and/or 
 *  modify it under the terms of the GNU General Public License 
 *  as published by the Free Software Foundation; either version 2 
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 *  General Public License for more details.
 *  
 *  You can receive a copy of the GNU General Public
 *  License at http://www.gnu.org/licenses/gpl.html
 *  
 *  Contributors:  
 *  	Imixs Software Solutions GmbH - initial API and implementation
 *  	Ralph Soika
 *******************************************************************************/
package org.imixs.workflow.jee.soap;

import java.util.Collection;
import java.util.Iterator;

import javax.annotation.security.DeclareRoles;
import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.jws.WebParam.Mode;

import org.imixs.workflow.ItemCollection;
import org.imixs.workflow.jee.ejb.EntityService;
import org.imixs.workflow.jee.ejb.WorkflowService;
import org.imixs.workflow.webservices.soap.WorkflowManagerService;
import org.imixs.workflow.xml.EntityCollection;
import org.imixs.workflow.xml.XMLItemCollection;
import org.imixs.workflow.xml.XMLItemCollectionAdapter;

/**
 * The WebService Implementation class uses the corresponding stateless session
 * ejb Implementation class. This class must match the corresponding WSDL file.
 * Therefore the necessary elements (name,targetNamespace,serviceName,portName)
 * are named in the WebService annotation. The WSDL file have to be supported in
 * the META-INF/wsdl directory. For basic authentification this EJB is defined
 * additional in the sun-ejb-jar.xml
 * 
 * @author Ralph Soika
 * @version 0.0.1
 */

@DeclareRoles( { "org.imixs.ACCESSLEVEL.NOACCESS",
		"org.imixs.ACCESSLEVEL.READERACCESS",
		"org.imixs.ACCESSLEVEL.AUTHORACCESS",
		"org.imixs.ACCESSLEVEL.EDITORACCESS",
		"org.imixs.ACCESSLEVEL.MANAGERACCESS" })
@RolesAllowed( { "org.imixs.ACCESSLEVEL.NOACCESS",
		"org.imixs.ACCESSLEVEL.READERACCESS",
		"org.imixs.ACCESSLEVEL.AUTHORACCESS",
		"org.imixs.ACCESSLEVEL.EDITORACCESS",
		"org.imixs.ACCESSLEVEL.MANAGERACCESS" })
// @Stateless
// @Remote(org.imixs.workflow.jee.ws.WorkflowWebService.class)
// @WebService(name = "WorkflowManagerService", targetNamespace =
// "http://imixs.org/workflow/services", serviceName = "WorkflowManagerService",
// portName = "WorkflowManagerServicePort", wsdlLocation =
// "ix_workflowservice.wsdl")
@WebService
public class WorkflowWebServiceBean implements WorkflowManagerService {

	@EJB
	EntityService entityService;

	@EJB
	WorkflowService workflowManager;

//	@Resource
//	SessionContext ctx;

	@WebMethod
	public XMLItemCollection getWorkItem(
			@WebParam(name = "workitemid", mode = Mode.IN) String uniqueid)
			throws Exception {
		ItemCollection ic = workflowManager.getWorkItem(uniqueid);
		return XMLItemCollectionAdapter.putItemCollection(ic);
	}

	@WebMethod
	public EntityCollection getWorkList(
			@WebParam(name = "key", mode = Mode.IN) String name,
			@WebParam(name = "startpos", mode = Mode.IN) int startpos,
			@WebParam(name = "count", mode = Mode.IN) int count)
			throws Exception {

		Collection<ItemCollection> col = workflowManager.getWorkList(name,
				startpos, count,null,0);
		return XMLItemCollectionAdapter.putCollection(col);
	}

	@WebMethod
	public EntityCollection getWorkListByRef(
			@WebParam(name = "key", mode = Mode.IN) String aref,
			@WebParam(name = "startpos", mode = Mode.IN) int startpos,
			@WebParam(name = "count", mode = Mode.IN) int count)
			throws Exception {

		Collection<ItemCollection> col = workflowManager.getWorkListByRef(aref,
				startpos, count,null,0);
		return XMLItemCollectionAdapter.putCollection(col);
	}

	@WebMethod
	public EntityCollection getWorkListByCreator(
			@WebParam(name = "key", mode = Mode.IN) String name,
			@WebParam(name = "startpos", mode = Mode.IN) int startpos,
			@WebParam(name = "count", mode = Mode.IN) int count)
			throws Exception {

		Collection<ItemCollection> col = workflowManager.getWorkListByCreator(
				name, startpos, count,null,0);
		return XMLItemCollectionAdapter.putCollection(col);
	}

	@WebMethod
	public EntityCollection getWorkListByGroup(
			@WebParam(name = "key", mode = Mode.IN) String name,
			@WebParam(name = "startpos", mode = Mode.IN) int startpos,
			@WebParam(name = "count", mode = Mode.IN) int count)
			throws Exception {

		Collection<ItemCollection> col = workflowManager.getWorkListByGroup(
				name, startpos, count,null,0);
		return XMLItemCollectionAdapter.putCollection(col);
	}

	@WebMethod
	public EntityCollection getWorkListByProcessID(
			@WebParam(name = "key", mode = Mode.IN) int aID,
			@WebParam(name = "startpos", mode = Mode.IN) int startpos,
			@WebParam(name = "count", mode = Mode.IN) int count)
			throws Exception {

		Collection<ItemCollection> col = workflowManager
				.getWorkListByProcessID(aID, startpos, count,null,0);
		return XMLItemCollectionAdapter.putCollection(col);
	}

	@WebMethod
	public XMLItemCollection processWorkItem(
			@WebParam(name = "workitem", mode = Mode.IN) XMLItemCollection workitem,
			@WebParam(name = "activityid", mode = Mode.IN) int activityid)
			throws Exception {
		ItemCollection ic = XMLItemCollectionAdapter
				.getItemCollection(workitem);
		ic = workflowManager.processWorkItem(ic);
		return XMLItemCollectionAdapter.putItemCollection(ic);
	}

	@WebMethod
	public void removeWorkItem(
			@WebParam(name = "workitemid", mode = Mode.IN) String uniqueid)
			throws Exception {
		ItemCollection itemCol = entityService.load(uniqueid);
		workflowManager.removeWorkItem(itemCol);
	}

	

}
